#!/usr/bin/python

# Copyright (C) 2009 Stefano Zacchiroli <zack@pps.jussieu.fr>
# License: GPL version 3 or (at your option) any later version

# Requirements:
# - python-matplotlib
# - python-numpy

# Sample usage:
#
#   $ ./history_graph.py -b debian-history -f png -f eps \
#       history/debian-history.summary
#
# which will generate graph files debian-history.{eps,png}

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import optparse
import sys

usage = """Usage: %prog [OPTIONS] DATA

DATA should be a textual record, line oriented, TAB-separated with
columns "release name", "number of packages", "number of direct deps",
"number of strong deps"
"""

supported_formats = set(['png', 'pdf', 'ps', 'eps', 'svg'])

def load_table(fname):
    """return a list of tuples, one for each column in the input table"""

    l = []
    for line in open(fname):
        if line.startswith('#'):
            continue
        row = line.split('\t')
        l.append([row[0]] + map(int, row[1:]))
    return l

def plot(data, bar_labels):
    ind = np.arange(len(data))	# number of releases
    width = 0.25
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.set_yscale('log')
    ax.set_xticks(ind+width)
    ax.set_xlabel("Debian release")
    ax.set_xlim(xmax=len(data))
    # ax.set_ylim(ymin=100)	# TODO: this doesn't work as expected :-(

    rels, pkgs, dir_deps, strong_deps = [], [], [], []
    for (r, p, d, s) in data:
        rels.append(r)
        pkgs.append(p)
        dir_deps.append(d)
        strong_deps.append(s)
    pkgs_bar = ax.bar(ind, pkgs, width, color='k')
    dir_deps_bar = ax.bar(ind+width, dir_deps, width, color='r')
    strong_deps_bar = ax.bar(ind+2*width, strong_deps, width, color='y')

    ax.set_xticklabels(map(lambda x: x.split()[1], rels))
    ax.legend((pkgs_bar[0], dir_deps_bar[0], strong_deps_bar[0]),
              ("Packages", "Direct dependencies", "Strong dependencies"),
              loc="upper left")

    return fig

def main():
    global supported_formats

    cmdline = optparse.OptionParser(usage=usage)
    cmdline.add_option("-i", "--interactive", action="store_true",
                       dest="interactive", default=False,
                       help="Interactive mode to browse the correlation graph")
    cmdline.add_option("-f", "--format", dest="format", default=["png"],
                       action="append",
                       help="output format (one of: png, pdf, ps, eps, svg), "
                       "can be given multiple times, default: 'png'")
    cmdline.add_option("-b", "--basename", dest="basename", default="corrgraph",
                       help="basename for graph output, default: 'corrgraph'")
    (options, args) = cmdline.parse_args()
    options.format = set(options.format)
    if len(args) < 1:
        cmdline.error("Must specify a data file")
    if not options.format.issubset(supported_formats):
        cmdline.error("Unsupported format(s): %s" %
                      list(options.format - supported_formats))
    if not options.interactive:
        matplotlib.use('Agg')

    data = load_table(args[0])
    fig = plot(data, bar_labels=['Packages', 'Direct dependencies',
                                 'Strong dependencies'])
    if options.interactive:
        plt.show()
    else:
        for fmt in set(options.format):
            fname = "%s.%s" % (options.basename, fmt)
            plt.savefig(fname, format=fmt)
            print fname

if __name__ == '__main__':
    main()
